/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Marat Garafutdinov
 * Student ID: 9825627
 * Section:    AE
 *	
 * STARTER PROGRAM
 *
 * To start with, your job is to figure out how this program works.
 *
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 
 *	 In this starter version, you can do much less.
 *   You can change the card background, place one shape on the card,
 *   and change the shape back and forth between two possible kinds.
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
	

#define TRUE    		1
#define FALSE   		0

/* ADD SHAPE   -- you click the card, and a new object is placed there.
 *                You get into this mode by clicking an ADD button.
 *                After doing an ADD, you are put into SELECTED mode.
 * SELECTED    -- An object is selected. In this mode, the type 
                  rotation applies to the current object.*/

#define STATUS_ADD_SHAPE 1
#define STATUS_SELECTED 2
#define STATUS_ADD_MSG 3
#define STATUS_SEEK 4

#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	100

/* change this if you add or delete buttons*/
#define NUM_BUTTONS 	6
#define SELECTED_COLOR  RED

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50) 

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* Types of objects we support.  (Perhaps not all types are used yet.)*/
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MSG 2

/* if you add new types, you may need to change this*/
#define MAX_SHAPES 4
#define MAX_MSG 5
#define FIRST_SHAPE_NUM 0
#define FIRST_MSG_NUM 0

/* These are the kinds of shapes we support.  You'll need to add more. */
#define SHAPE_KENNY  0
#define SHAPE_STAR   1
#define SHAPE_CIRCLE 2
#define SHAPE_RECT   3

#define MESSAGE_1 0 
#define MESSAGE_2 1
#define MESSAGE_3 2
#define MESSAGE_4 3
#define MESSAGE_5 4

/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area, not on a button*/	


#define SHAKE_LNG 3 /* For drawing Kenny */

/* For drawing a star */ 
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416

/* This is the maximum number of objects you can place on the card
   It is also the base for all of the arrays, that enable having multiple
   objects*/
#define MAX_OBJECTS 15

/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);

void DrawButtons(int last_button);

void DrawCard (int background_color, int status, int object_count, 
			   int object_type[MAX_OBJECTS], int object_num[MAX_OBJECTS], 
			   int object_color[MAX_OBJECTS], int object_x[MAX_OBJECTS], 
			   int object_y[MAX_OBJECTS]);

void DrawShape(int shape_type[MAX_OBJECTS], int color[MAX_OBJECTS], 
			   int bottom_x[MAX_OBJECTS], int bottom_y[MAX_OBJECTS], 
			   int *i);

void DrawKenny (int x, int y, int color);

void DrawCircle (int x, int y, int color);

void DrawRect (int x, int y, int color);

void DrawStar (int x, int y, int color);

void Msg1 (int x, int y, int color);
void Msg2 (int x, int y, int color);
void Msg3 (int x, int y, int color);
void Msg4 (int x, int y, int color);
void Msg5 (int x, int y, int color);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[MAX_OBJECTS], 
					  int object_num[MAX_OBJECTS], int object_color[MAX_OBJECTS],					   
					  int object_x[MAX_OBJECTS], int object_y[MAX_OBJECTS], 
					  int *last_button, int *current);

int ClassifyMouseClick(int mouse_x, int mouse_y);

void HandleButton(int x, int y, int *background_color, int *status,
				  int object_count, int *object_type, int *object_num, 
				  int *object_color, int *last_button, int *current);

/* Add a shape to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			  int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int *current);

void AddMsg(int x, int y, int *status, int *object_count, 
			  int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int *current);



void MoveShape(int x,  int y, int *status, int *object_x,int *object_y);

void DrawMsg (int msg_type[MAX_OBJECTS], int color[MAX_OBJECTS], 
			   int bottom_x[MAX_OBJECTS], int bottom_y[MAX_OBJECTS], 
			   int *i);


void ObjectSeek(int *status, int object_x[MAX_OBJECTS], 
				int object_y[MAX_OBJECTS], int *object_count, 
				int object_type[MAX_OBJECTS], int object_num[MAX_OBJECTS],
				int *mouse_x, int *mouse_y, int *current);


/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 * initialize them, and to contain the "event loop," which constantly looks 
 * to see if an event has happened, and then processes the event by calling 
 * an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	
	int background_color = BLUE;
	int status = STATUS_ADD_SHAPE;
	int object_count = 0;     
	int object_type[MAX_OBJECTS];                   
	int object_num[MAX_OBJECTS];       
	int object_color[MAX_OBJECTS];     
	int object_x[MAX_OBJECTS];         
	int object_y[MAX_OBJECTS];
	int current = object_count;


	int last_button = -1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */

	int event; /*the most recent GP142 event*/
	
	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    
	

	/* These GP142 initializations don't need to be changed */
	
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	
	while (!quit) /* this is the famous "event loop" */
	
	{  
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		
		DrawCard (background_color, status, object_count, 
			      object_type, object_num, object_color, object_x, object_y);
		
		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
			/*not needed for this program. Use this if user will need to 
			  hit a key. Note that scanf CANNOT be used with GP142.  If you 
			  use this, the key character value is in variable "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
					         &object_count, object_type, 
							 object_num, object_color, 
							 object_x, object_y, &last_button, &current);			
			break;
			
		case GP142_PERIODIC:
			
			/*An events has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
/* To add buttons, you'll need to change the #defined constant NUM_BUTTONS
 * and add messages to this array.  Although we haven't covered 2-D arrays
 * yet, you should be able to play with this enough to get it going.
 * Look at course packet, page N-25, for a similar example.
 */
	char b_labels [NUM_BUTTONS][25] = {
		" Add shape",
		" Add message",
		" Background color",
		" Object color",
		" Next",
		" Select"
	};
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(1,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 0);
		
		GP142_rectangleXY(22,
			-GP142_XMAX + 10,
			y - 10,
			-GP142_XMAX + BUTTON_WIDTH + 10,
			y - BUTTON_HEIGHT - 10, 0);
		
		GP142_rectangleXY(23,
			-GP142_XMAX + 6,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH + 5,
			y - BUTTON_HEIGHT , 0);
		
								
		label_y = y - BUTTON_HEIGHT/2 - 10;
		
		
		if (i == last_button)
		
		GP142_printfXY(SELECTED_COLOR, text_offset, label_y, font_size, b_labels[i]);
		else
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);	
	}	



}


/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[MAX_OBJECTS], int object_num[MAX_OBJECTS], 
			   int object_color[MAX_OBJECTS], int object_x[MAX_OBJECTS], 
			   int object_y[MAX_OBJECTS])
{
	int font_size = 14;
	int i; 
			
	/*
	 * Draw the background of the card.
	 */
	GP142_rectangleXY (background_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	
	/* 
	 * Draw a shape on the card, if one currently exists.
	 */
	
	
	for (i = 0; i < MAX_OBJECTS; i = i + 1)	
	{
		if ((object_type[i] == OBJECT_SHAPE) && (object_count > 0))  
			
				{ 					
					DrawShape (object_num, object_color, object_x, object_y, &i);
				}
		
	
		if ((object_type[i] == OBJECT_MSG) && (object_count > 0)) 
				
				{ 					
					DrawMsg (object_num, object_color, object_x, object_y, &i);
				}
	
	}
	
	
	
	
	/*
	 * Write an appropriate status message below the background
	 */
	switch (status) {
	
	case STATUS_ADD_SHAPE:
		
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape");
		break;
	
	case STATUS_SELECTED:
		
		if (object_count > 0)
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"Object selected. Click button to change shape.");
		else 
		
			GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Place shape on card first, then you can change it.");
		break;
	
	case STATUS_ADD_MSG:
				
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a message");
		break;
		
	case STATUS_SEEK:
				
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click close to object's origin to select it");
		break;
	
	
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;

	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw one shape.
 */
void DrawMsg (int msg_type[MAX_OBJECTS], int color[MAX_OBJECTS], 
			   int bottom_x[MAX_OBJECTS], int bottom_y[MAX_OBJECTS], 
			   int *i)

{
    

	switch ((msg_type[*i] - FIRST_MSG_NUM) % MAX_MSG) 
	
	{
	
	case MESSAGE_1:
		Msg1 (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	case MESSAGE_2:
		Msg2 (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	case MESSAGE_3:
		Msg3 (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	case MESSAGE_4:
		Msg4 (bottom_x[*i], bottom_y[*i], color[*i]);
		break;

	case MESSAGE_5:
		Msg5 (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}




void DrawShape(int shape_type[MAX_OBJECTS], int color[MAX_OBJECTS], 
			   int bottom_x[MAX_OBJECTS], int bottom_y[MAX_OBJECTS], 
			   int *i) 

{
    

	
	switch ((shape_type[*i] - FIRST_SHAPE_NUM) % MAX_SHAPES) 
	
	{
	
	case SHAPE_KENNY:
		DrawKenny (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	case SHAPE_STAR:
		DrawStar (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	case SHAPE_CIRCLE:
		DrawCircle (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	case SHAPE_RECT:
		DrawRect (bottom_x[*i], bottom_y[*i], color[*i]);
		break;
	
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}

/* Message - drawing functions */

void Msg1 (int x, int y, int color)

{
	int h = (rand () % 24);
	GP142_printfXY (color + h, x, y, 12,
	"OHMYGOD, THEY'VE KILLED KENNY! BASTARDS!!!");
}


void Msg2 (int x, int y, int color)

{
	int h = (rand () % 4);
	GP142_printfXY (color, x + h, y - h, 12,
			"What da' hell are you people doing here ?");
}


void Msg3 (int x, int y, int color)

{
	int h = (rand () % 4);
	GP142_printfXY (color, x, y, 12 - h,
		"Ok, people move on, nothing to see here!");
}


void Msg4 (int x, int y, int color)

{
	int h = (rand () % 4);
	
	GP142_printfXY (color, x - h, y, 12,
			"I'll kick you in the ");
	GP142_printfXY (color, x + 89, y, 12 + h,"NUTS!");
}


void Msg5 (int x, int y, int color)

{
	int h = (rand () % 4);
	GP142_printfXY (color, x, y, 12,
			"Be my Valentine, my       honeybunch!    ");
	GP142_printfXY (color + h, x + 101, y, 12 + h,
			" =>");
	GP142_printfXY (color + h, x + 181, y, 12 + h,
			"  <=");
}

/*
 * Draw Kenny on the card. The color of Kenny's 
 * coat is changeable.
 */
void DrawKenny (int x, int y, int color) {

	int h = (SHAKE_LNG/2) + (rand () %4);
	
	/* Kenny's body */
	GP142_triangleXY(color, x, y, x - 18, y - 35, x + 18, y - 35, 0); 
	GP142_lineXY(0, x, y - 22, x, y - 35, 1); 
	GP142_lineXY(0, x - 18, y - 35, x + 18, y - 35, 1);
	
	GP142_lineXY(0, x , y, x + 18, y - 35, 1);
	GP142_lineXY(0, x, y, x - 18, y - 35, 1);
	
	GP142_rectangleXY(color, x - 16, y - 36, x + 16, y - 40, 0);
	GP142_rectangleXY(0, x - 18, y - 40, x + 18, y - 42, 0);

	/* Hands */
	GP142_circleXY (14, x - 14, y - 30, 4);
	GP142_circleXY (14, x + 14, y - 30, 4);

	/*Kenny's hat */
	GP142_circleXY (0, x, y - h, 23);
	GP142_circleXY (color, x, y - h, 22);
	GP142_circleXY (14, x, y - h, 15);
	GP142_ovalXY (16, x + 8, y + 14 - h ,x - 8, y - 12 - h, 0);
	
	/* Knot under Kenny's chin */
	GP142_circleXY (0, x, y - 14 - h, 2);
	GP142_lineXY(0, x, y - 14 - h, x - 3, y - 19 - h, 1);
	GP142_lineXY(0, x, y - 14 - h ,x + 3, y - 19 - h, 1);

	/* Kenny's eye sockets */
	GP142_ovalXY (0, x + 8 ,y + 8 - h ,x - 1 ,y - 8 - h, 0);
	GP142_ovalXY (0, x - 8 ,y + 8 - h,x + 1 ,y - 8 - h, 0);
	
	/* Kenny's eyes */
	GP142_ovalXY (1, x + 7, y + 7 - h ,x ,y - 7 - h, 0);
	GP142_ovalXY (1, x - 7 , y + 7 - h ,x ,y - 7 - h, 0);
	
	/* Kenny's pupils */ 
	GP142_circleXY (0, x - 2, y + 2 - h, 1);
	GP142_circleXY (0, x + 2, y + 2 - h, 1);
	

}



void DrawStar (int x, int y, int color) {

	GP142_circleXY (0, x, y , 23);
	GP142_circleXY (color, x, y , 22);
	GP142_circleXY (14, x, y , 15);
	GP142_ovalXY (16, x + 8, y + 14  ,x - 8, y - 12 , 0);
	
	/* Knot under Kenny's chin */
	GP142_circleXY (0, x, y - 14 , 2);
	GP142_lineXY(0, x, y - 14, x - 3, y - 19, 1);
	GP142_lineXY(0, x, y - 14, x + 3, y - 19, 1);

	/* Kenny's eye sockets */
	GP142_ovalXY (0, x + 8 ,y + 8, x - 1 ,y - 8, 0);
	GP142_ovalXY (0, x - 8 ,y + 8, x + 1 ,y - 8, 0);
	
	/* Kenny's eyes */
	GP142_ovalXY (1, x + 7, y + 7, x ,y - 7, 0);
	GP142_ovalXY (1, x - 7 , y + 7, x ,y - 7, 0);
	
	
	GP142_printfXY (0, x - 6, y - 6, 11,
	"+");
	GP142_printfXY (0, x + 2, y - 6, 11,
	"+");
}

void DrawCircle(int x, int y, int color)
{
	int h = (rand () %4);
	GP142_ovalXY(color, x - 10 , y - 10, x + 10, y + 10, 1);
	GP142_ovalXY(color, x - 30, y - 30, x + 30, y + 30, 1);
	GP142_lineXY(color, x - 40, y, x - 10, y, 1); 
	GP142_lineXY(color, x + 40, y, x + 10, y, 1);
	GP142_lineXY(color, x, y - 40, x, y - 10, 1); 
	GP142_lineXY(color, x, y + 40, x, y + 10, 1);

}


void DrawRect (int x, int y, int color)
{
	int h = (rand () %24);
	GP142_circleXY(GREEN, x - 15  , y + 10, 25);
	GP142_rectangleXY(color, x + 11, y + 10 , x - 41 , y - 100 , 0);
	GP142_rectangleXY(GREEN, x + 8, y + 7, x - 38 , y - 97, 0);
	GP142_rectangleXY(RED, x - 38, y - 10 , x + 8 , y - 25, 0);
	GP142_printfXY (h, x - 38, y - 20, 7,
	"Honey Bucket");
	GP142_circleXY(BLACK, x + 8, y - 50, 4);
}

/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    
	  return CLICK_BUTTON;
  
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  
  else
  return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[MAX_OBJECTS], 
					  int object_num[MAX_OBJECTS], int object_color[MAX_OBJECTS], 
					  int object_x[MAX_OBJECTS], int object_y[MAX_OBJECTS], 
					  int *last_button, int *current)
{

	int mouse_click;
		
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
		
	if (mouse_click == CLICK_BUTTON) {
				
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change */
		HandleButton(mouse_x, mouse_y, background_color, status, 
					 *object_count, &object_type[*current], &object_num[*current], 
					 &object_color[*current], last_button, current);

	} else 
		
		if ((mouse_click == CLICK_CARD) && (*status == STATUS_SELECTED) && (*current == *object_count))
			
			{
				MoveShape(mouse_x, mouse_y, status, &object_x[*current-1], 
					&object_y[*current-1]);
			}
			
		
		if ((mouse_click == CLICK_CARD) && (*status == STATUS_ADD_MSG))
			
			{
		AddMsg(mouse_x, mouse_y, status, object_count, 
			   &object_type[*current], &object_num[*current], 
			   &object_color[*current], &object_x[*current], 
			   &object_y[*current], current); 

			}
			
		if ((mouse_click == CLICK_CARD) && (*status == STATUS_SEEK))
			
			{
		
		ObjectSeek( status, object_x, object_y, object_count, 
					object_type, object_num,
					&mouse_x, &mouse_y, current);

			}
		
			
			
	if ((mouse_click == CLICK_CARD) && (*status == STATUS_ADD_SHAPE))
		{	 
			AddShape(mouse_x, mouse_y, status, object_count, 
			   &object_type[*current], &object_num[*current], 
			   &object_color[*current], &object_x[*current], 
			   &object_y[*current], current);
		}	
	
	
			
		
		/* Ignore any other mouse clicks */
}

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton(int x, int y, int *background_color, int *status,
				  int object_count, int *object_type, int *object_num,
				  int *object_color, int *last_button, int *current)
{
	int buttonNum = 0;
	
	
	*status = STATUS_SELECTED;

	/* If we're outside of this area, we could not have clicked on
	 a button.	*/

	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
					
	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	
	case 0:
		 *status = STATUS_ADD_SHAPE; /* Add new shape */ 
		 *last_button = 0;
		 break; 
	
	case 1 :

		 *status = STATUS_ADD_MSG;
		 *last_button = 1;	
		 break;		

	case 2 :
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		*last_button = 2;
		break;
	
	case 3:
		/* Change current object color */  
		 
		*object_color = (*object_color + 1) % MAX_COLORS; 
		*last_button = 3;		
		break; 

	case 4:
		/* Change the type of the currently selected object */
		
		if ((*status == STATUS_SELECTED)) 
		{	
			if (*object_type == OBJECT_SHAPE)
			*object_num = (*object_num + 1) % MAX_SHAPES;
		
			if (*object_type == OBJECT_MSG)
	    	*object_num = (*object_num + 1) % MAX_MSG;		
		}

		*last_button = 4;
		break;
	
	case 5:
		*status = STATUS_SEEK; 
		*last_button = 5;
		break;
    	
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	
	}
}


/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int *current) 
{
		if (*status == STATUS_ADD_SHAPE)
		
		{		   		
			*status = STATUS_SELECTED;
			*object_count = *object_count + 1;
			*object_type = OBJECT_SHAPE;
 			*object_num = SHAPE_KENNY;
			*object_color = 13; 
			*object_x = x;
			*object_y = y;
			*current = *object_count;
		}
		
		
}

void AddMsg(int x, int y, int *status, int *object_count, 
			  int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int current[MAX_OBJECTS])
{
	if (*status == STATUS_ADD_MSG)
		
		{		   		
			*status = STATUS_SELECTED;
			*object_count = *object_count + 1;
			*object_type = OBJECT_MSG;
 			*object_num = MESSAGE_1;
			*object_color = 0; 
			*object_x = x;
			*object_y = y;
			*current = *object_count;
			
		}
	
}


void MoveShape(int x,  int y, int *status, int *object_x,int *object_y)

{
	*status = STATUS_SELECTED;
		*object_x = x;
		*object_y = y;

}


void ObjectSeek(int *status, int object_x[MAX_OBJECTS], 
				int object_y[MAX_OBJECTS], int *object_count, 
				int object_type[MAX_OBJECTS], int object_num[MAX_OBJECTS],
				int *mouse_x, int *mouse_y, int *current)
{
	int i;
	for (i = MAX_OBJECTS - 1; i >= 0; i = i - 1)
		
		{
			if ((abs(object_x[i] - *mouse_x) < 15) &&  (abs(object_y[i] - *mouse_y) < 15))				
				{
					*status = STATUS_SELECTED;
					*current = i ;
					break;				
				}
		
		}
}
	
	
	

